/*
  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.

  Author: Volker Neff

*/
#ifndef Field3D_t_INL
#define Field3D_t_INL

#include "field.h"
#include "field_operation.h"

inline constexpr Field3D_t::Field3D_t() noexcept
    : _var_()
{
}

inline Field3D_t::Field3D_t(Field3D_t& other)
    : _var_(other)
{
}

template< class T >
inline constexpr Field3D_t::Field3D_t( T&& t ) noexcept
    : _var_(std::move(t))
{
}

template<   class T,
            class... Args >
inline constexpr Field3D_t::Field3D_t( std::in_place_type_t<T>,
                        Args&&... args )
    : _var_(std::in_place_type_t<T>{}, args...)
{
}

template< class T,
        class U,
        class... Args >
inline constexpr Field3D_t::Field3D_t(std::in_place_type_t<T>,
                        std::initializer_list<U> il,
                        Args&&... args )
    : _var_(std::in_place_type_t<T>{}, il, args...)
{
}

template< std::size_t I,
        class... Args >
inline constexpr Field3D_t::Field3D_t( std::in_place_index_t<I>,
                        Args&&... args )
    : _var_(std::in_place_index_t<I>{}, args...)
{
}

template< std::size_t I,
        class U,
        class... Args >
inline constexpr Field3D_t::Field3D_t( std::in_place_index_t<I>,
                        std::initializer_list<U> il,
                        Args&&... args )
    : _var_(std::in_place_index_t<I>{}, il, args...)
{
}

// Overwrites of the std::variant<> operator=
inline Field3D_t& Field3D_t::operator=( const Field3D_t& rhs ) {
    _var_::operator=(rhs);
    return *this;
}

inline Field3D_t& Field3D_t::operator=( Field3D_t&& rhs ) noexcept {
    _var_::operator=(std::move(rhs));
    return *this;
}

template< class T >
inline Field3D_t& Field3D_t::operator=( T&& t ) noexcept {
    _var_::operator=(std::move(t));
    return *this;
}

inline void Field3D_t::resize(size_t count) {
    field_operation([](LambdaField3D_T_ field, size_t count){field.resize(count);}, *this, count);
}

template<typename U>
inline void Field3D_t::resize(size_t count, const U value) {
    field_operation([](LambdaField3D_T_ field, size_t count, const U value){field.resize(count, value);}, *this, count, value);
}

inline void Field3D_t::setMissValue(double value) {
    if(std::holds_alternative<Field3DFloat>(*this)) {
        std::get<Field3DFloat>(*this).setMissValue(value);
    } else if(std::holds_alternative<Field3DDouble>(*this)) {
        std::get<Field3DDouble>(*this).setMissValue(value);
    } else {
        throw std::runtime_error("Not supported Type");
    }
}

inline double Field3D_t::getMissValue() const {
    return field_operation([](LambdaField3D_T_ field){return field.getMissValueDouble();}, *this);
}

#endif // Field3D_t_INL