/*
  This file is part of CDO. CDO is a collection of Operators to manipulate and analyse Climate model Data.

  Author: Uwe Schulzweida

*/
#ifndef FIELD_FUNCTIONS_H
#define FIELD_FUNCTIONS_H

#include "field.h"

enum FieldFunc
{
  FieldFunc_Min = 100,
  FieldFunc_Max,
  FieldFunc_Range,
  FieldFunc_Sum,
  FieldFunc_Avg,
  FieldFunc_Mean,
  FieldFunc_Var,
  FieldFunc_Var1,
  FieldFunc_Std,
  FieldFunc_Std1,
  FieldFunc_Skew,
  FieldFunc_Kurt,
  FieldFunc_Median,
  FieldFunc_Pctl,

  FieldFunc_Cor,
  FieldFunc_Covar,
  FieldFunc_Avgw,
  FieldFunc_Meanw,
  FieldFunc_Stdw,
  FieldFunc_Std1w,
  FieldFunc_Varw,
  FieldFunc_Var1w,
  FieldFunc_Minidx,
  FieldFunc_Maxidx,
  FieldFunc_Rmsd,

  FieldFunc_Add,
  FieldFunc_Sub,
  FieldFunc_Mul,
  FieldFunc_Div,
  FieldFunc_Mod,

  FieldFunc_Atan2,
  FieldFunc_Setmiss,
};


// fieldmem.cc
void fields_from_vlist(int vlistID, FieldVector2D &field2D);
void fields_from_vlist(int vlistID, FieldVector2D &field2D, int ptype);
void fields_from_vlist(int vlistID, FieldVector2D &field2D, int ptype, double fillValue);

// field.cc
double field_function(const Field_t &field, int function);

template<typename T>
double field_function(const Field<T> &field, int function);


double field_min(const Field_t &field);
double field_range(const Field_t &field);
double field_max(const Field_t &field);
double field_sum(const Field_t &field);
double field_mean(const Field_t &field);
double field_meanw(const Field_t &field);
double field_avg(const Field_t &field);
double field_avgw(const Field_t &field);
double field_std(const Field_t &field);
double field_std1(const Field_t &field);
double field_var(const Field_t &field);
double field_var1(const Field_t &field);
double field_stdw(const Field_t &field);
double field_std1w(const Field_t &field);
double field_varw(const Field_t &field);
double field_var1w(const Field_t &field);
double field_skew(const Field_t &field);
double field_kurt(const Field_t &field);
double field_median(const Field_t &field);

// ENS validation
double field_rank(Field_t &field);

double field_pctl(Field_t &field, double pn);

// field_zonal.cc
void zonal_function(const Field_t &field1, Field_t &field2, int function);
void zonal_min(const Field_t &field1, Field_t &field2);
void zonal_max(const Field_t &field1, Field_t &field2);
void zonal_range(const Field_t &field1, Field_t &field2);
void zonal_sum(const Field_t &field1, Field_t &field2);
void zonal_avg(const Field_t &field1, Field_t &field2);
void zonal_mean(const Field_t &field1, Field_t &field2);
void zonal_std(const Field_t &field1, Field_t &field2);
void zonal_std1(const Field_t &field1, Field_t &field2);
void zonal_var(const Field_t &field1, Field_t &field2);
void zonal_var1(const Field_t &field1, Field_t &field2);
void zonal_skew(const Field_t &field1, Field_t &field2);
void zonal_kurt(const Field_t &field1, Field_t &field2);
void zonal_median(const Field_t &field1, Field_t &field2);
void zonal_pctl(const Field_t &field1, Field_t &field2, double pn);

// field_meridional.cc
void meridional_function(const Field_t &field1, Field_t &field2, int function);
void meridional_pctl(const Field_t &field1, Field_t &field2, double pn);

void field_rms(const Field_t &field1, const Field_t &field2, Field_t &field3);


// fieldc.cc
void fieldc_function(Field_t &field, double rconst, int function);

template<typename T>
void fieldc_function(Field<T> &field, double rconst, int function);

void fieldc_mul(Field_t &field, double rconst);
void fieldc_div(Field_t &field, double rconst);
void fieldc_add(Field_t &field, double rconst);
void fieldc_sub(Field_t &field, double rconst);
void fieldc_min(Field_t &field, double rconst);
void fieldc_max(Field_t &field, double rconst);
void fieldc_mod(Field_t &field, double divisor);

// fieldc_complex.cc
void fieldc_function_complex(Field_t &field, const double rconstcplx[2], int function);

template<typename T>
void fieldc_function_complex(Field<T> &field, const double rconstcplx[2], int function);


// field2.cc
void field2_function(Field_t &field1, const Field_t &field2, int function);

template<typename T>
void field2_function(Field<T> &field1, const Field<T> &field2, int function);

void field2_add(Field_t &field1, const Field_t &field2);
void field2_sum(Field_t &field1, const Field_t &field2);
void field2_sub(Field_t &field1, const Field_t &field2);
void field2_mul(Field_t &field1, const Field_t &field2);
void field2_div(Field_t &field1, const Field_t &field2);
void field2_min(Field_t &field1, const Field_t &field2);
void field2_max(Field_t &field1, const Field_t &field2);
void field2_atan2(Field_t &field1, const Field_t &field2);

void field2_sumq(Field_t &field1, const Field_t &field2);
void field2_sumw(Field_t &field1, const Field_t &field2, double w);
void field2_sumqw(Field_t &field1, const Field_t &field2, double w);
void field2_sumtr(Field_t &field1, const Field_t &field2, double refval);
void field2_vinit(Field_t &field1, const Field_t &field2);
void field2_vincr(Field_t &field1, const Field_t &field2);
void field2_vinit(Field_t &field1, const Field_t &field2, int vinit);
void field2_vincr(Field_t &field1, const Field_t &field2, int vincr);

void field2_sumsumq(Field_t &field1, Field_t &field2, const Field_t &field3);
void field2_maxmin(Field_t &field1, Field_t &field2, const Field_t &field3);
void field2_minidx(Field_t &field1, Field_t &field2, const Field_t &field3, int idx);
void field2_maxidx(Field_t &field1, Field_t &field2, const Field_t &field3, int idx);
void field2_var(Field_t &field1, const Field_t &field2, const Field_t &field3, int divisor);
void field2_std(Field_t &field1, const Field_t &field2, const Field_t &field3, int divisor);
void fieldc_var(Field_t &field1, const Field_t &field2, int nsets, int divisor);
void fieldc_std(Field_t &field1, const Field_t &field2, int nsets, int divisor);
void field2_moq(Field_t &field1, const Field_t &field2);
void field2_moqw(Field_t &field1, const Field_t &field2, double w);

void field2_count(Field_t &field1, const Field_t &field2);

// field2_complex.cc
void field2_function_complex(Field_t &field1, const Field_t &field2, int function);

template<typename T>
void field2_function_complex(Field<T> &field1, const Field<T> &field2, int function);

#endif /* FIELD_FUNCTIONS_H */